{$A-,B-,D+,E+,F-,G-,I+,L+,N-,O-,P+,Q-,R+,S+,T-,V-,X+}
{$M 16384,0,655360}

program PKTclean;

{
A FIDOnet utility.  Cleans garbage kludges from PKT files.
Required: TURBO Pascal 7.0 & Turbo Professional 5.11+.
Written by Dimmy Timchenko, 2:469/15
}

uses DOS, TPString;

const
	ID_STR = 'PKT Cleaner v.0.98.gamma. Freeware.';

const
	BUFSIZ = 40*1024;		{Size of file buffer}
	NAMSIZ = 512;			{Size of filename array}
	KL_SIZ = 51;			{Maximum length of kludge}
	KL_LEN = 150;			{Maximum number of kludges}
	AR_SIZ = 51;			{ ... AREAS }
	AR_LEN = 50;
	RP_SIZ = KL_SIZ;		{ ... REPLACES }
	RP_LEN = 25;
	TAB = #09;
	DLM = [' ',TAB,','];


const
	AREA_ID_SIZ = 5;
	AREA_ID : string[AREA_ID_SIZ] = 'AREA:';
	NetScan : boolean = FALSE;

var
	KludMax 	: byte;			{Number of kludges defined in CFG file}
	KLUDGES 	: array [1..KL_LEN] of string[KL_SIZ-1];
	AreaMax 	: byte;			{Number of areas defined in CFG file}
	AREAS   	: array [1..AR_LEN] of string[AR_SIZ-1];
	RPLSTRINGS 	: array [1..RP_LEN] of string[RP_SIZ-1];
	RplMax 		: byte;
	REPLACES 	: array [1..KL_LEN] of byte;

var
	PATH, LogFile		: string[64];
	TmpS				: string;
	F, FO, FL			: file;
	IOr      			: integer;
	S        			: SearchRec;

	i, n, tmpw, Percent,
	Actual, ActBak, shift 		: word;

	Cleaned, LogOpen, Exclude, Replaced,
	AddTag, MaskSpecified 		: boolean;

	Position, Diff, TotSrc,
	TotDst, TotDiff 			: longint;

var			{ Buffer for data processing }
	Buf : array [0..BUFSIZ-1] of byte;
			{ List of filenames to be processed }
	Names : array [1..NAMSIZ] of string[12];
	NameIdx, NameMax : word;

type
TFileRec = record			{ TPASCAL file record }
	Handle: Word;
	Mode: Word;
	RecSize: Word;
	Private: array[1..26] of Byte;
	UserData: array[1..16] of Byte;
	Name: array[0..79] of Char;
end;

var
	TagSet : set of char;
	Tag : char;

var
	HomePath 	: DirStr;
	HomeName	: NameStr;
	HomeExt		: ExtStr;

const
	StrictCheck  : boolean = FALSE;
	IfmailBugFix : boolean = FALSE;

	{Converts integer to string}

function Int2Str(I: Longint): String;
var
	S: string[11];
begin
	Str(I, S);
	Int2Str := S;
end;

	{Handles system log}

procedure Log ( Tag:char; Message:string );
var
	Year, Month, Day, DayOfWeek  : Word;
	Hour, Minute, Second, Sec100 : Word;
	MsgS : string;

	procedure Paste ( N:word; d:char );
	var
		DigS : string[4];
	begin
		Str ( N:2, DigS );
		if DigS[1]=' ' then DigS[1] := '0';
		MsgS := MsgS+d+DigS;
	end;

begin
	if LogFile = '' then Exit;
	if not LogOpen then begin
{$I-}
		Assign (FL, LogFile);
		Reset (FL,1);
		if IOresult <> 0 then begin
			Rewrite(FL,1);
			if IOresult <> 0 then begin
				WriteLn ( 'ERROR : Can''t open LogFile "', LogFile, '"');
				WriteLn ( 'Log disabled');
				LogFile := '';
				Exit;
			end;
		end else begin
			Seek(FL,FileSize(FL));
			if IOresult <> 0 then begin
				WriteLn ( 'INTERNAL error : can''t seek to the end of LogFile' );
				Close(FL);
				Halt(1);
			end;
		end;
		LogOpen := TRUE;
	end;
	GetDate( Year, Month, Day, DayOfWeek );
	GetTime( Hour, Minute, Second, Sec100 );
	MsgS := Tag;
	Paste ( Day,    ' ');
	Paste ( Month,  '.');
	Paste ( Hour,   ' ');
	Paste ( Minute, ':');
	Paste ( Second, ':');
	MsgS := MsgS + ' PKTCLEAN ' + Message + #$0D#$0A;
	BlockWrite (FL, MsgS[1], Length(MsgS) );
	if IOResult<>0 then begin
		WriteLn ( 'INTERNAL ERROR: Can''t write log' );
		Close (FL); IOr := IOResult;
		LogOpen := FALSE; LogFile := '';
	end;
{$I+}
end;

procedure CloseLog;
begin
	if not LogOpen then Exit;
{	Log ( '+', '--- end' ); }
	Close (FL);
end;

	{Displays error message}

procedure ErrHalt ( Etx:string );
begin
	WriteLn (Etx);
	Log ( '!', Etx );
	CloseLog;
	Halt(2);
end;

	{Displays warning message}

procedure Warn ( Etx:string );
begin
	WriteLn (Etx);
	Log ( '*', Etx );
end;

	{Displays warning message only ONCE}

procedure WarnOnce ( Etx:string );
var
	F : file;
	TimeWrn, TimeCfg : longint;
const
	Checked : boolean = FALSE;
	DoWarn  : boolean = FALSE;

begin
	if not Checked then begin;
		Checked := TRUE;
	{$I-}
		Assign (F, HomePath+HomeName+'.WRN' );
		Reset (F);
		if IOresult = 0 then begin
			GetFTime ( F, TimeWrn );
			Close(F);
			Assign (F, HomePath+HomeName+'.CFG' );
			Reset (F);
			if IOresult = 0 then begin
				GetFTime ( F, TimeCfg );
				Close(F);
				if TimeWrn >= TimeCfg then Exit;
			end;
		end;
		Rewrite (F);
		Close (F);
		DoWarn := TRUE;
	{$I+}
	end;
	WriteLn (Etx);
	Log ( '!', Etx );
end;

	{Displays information message}

procedure Inform ( Etx:string );
begin
	WriteLn (Etx);
	Log ( ':', Etx );
end;

	{ Scans directory and fills list of files }

procedure ScanDIR ( Mask:string );
begin
	FindFirst (PATH+Mask, Archive, S);
	if DOSError <> 0 then Exit;

	while DOSError = 0 do begin
		Write ( #$0D+S.Name );
		Inc(NameMax);
		if NameMax > NAMSIZ then begin
			WriteLn;
			ErrHalt ( 'More than '+Int2Str(NAMSIZ)+' files in directory' );
		end;
		Names[NameMax] := S.Name;
		FindNext ( S );
	end;
end;

	{ Checks IOresult and halts with a scream }

procedure IOcheck ( var F: file );
const
	ERR_SIZ = 14;
	DosErrors : array[1..ERR_SIZ] of record
		cod : byte;
		msg : string[19];
	end =
	(
		 ( cod : 2;   msg : 'File not found' ),
		 ( cod : 3;   msg : 'Path not found' ),
		 ( cod : 4;   msg : 'Too many open files' ),
		 ( cod : 5;   msg : 'Access defined' ),
		 ( cod : 6;   msg : 'Bad file handle' ),
		 ( cod : 12;  msg : 'Bad access code' ),
		 ( cod : 15;  msg : 'Bad drive number' ),
		 ( cod : 18;  msg : 'No more files' ),
		 ( cod : 100; msg : 'Disk read error' ),
		 ( cod : 101; msg : 'Disk write error' ),
		 ( cod : 103; msg : 'File not open' ),
		 ( cod : 104; msg : 'Not open for input' ),
		 ( cod : 105; msg : 'Not open for output' ),
		 ( cod : 0;   msg : '' )
);
var
	IOr, i : integer;
	msg : string[21];

begin
	IOr := IOresult;
	if IOr = 0 then Exit;
	for i:=1 to ERR_SIZ do
		if DosErrors[i].cod = IOr then break;
	if i < ERR_SIZ then
		msg := ': '+ DosErrors[i].msg
	else
		msg := '#' + Int2Str(IOr);

	ErrHalt ( 'DOS I/O Error '+msg+'. File '+TFileRec(F).Name);

end;

	{ Scans buffer looking for kludges and deletes found "strings" }

procedure ScanBuf; assembler;
var
	ScanOn, Plus : byte;
	StrSiz, AfterFND_SI, AfterFND_CX : word;
asm
		mov		si,offset Buf			{ Scanning [SI] }
		mov		di,si					{ Copying to [DI] }
		cld
		mov		cx,Actual				{ Number of bytes in buffer }
		mov		Plus,0

		mov		ax, ds
		mov		es, ax
		mov		al, NetScan
		mov		ScanOn,al

@1:		lodsb							{ Find CR - "line" delimiter or 00 }
@1a:	stosb							{ & copy src --> dst }
		or		al,al					{ 00 : look for AREA:}
		jz		@ChkArea
		cmp		al,$0D					{ CR}
@1b:	loopnz	@1
		jcxz	@Exit					{ End of array }

		lodsb
		cmp		al,$01					{ ^A ? Kludge prefix }
		jnz		@1a

		test	ScanOn,$FF				{ Kludge scan turned off? }
		jz		@1a

		mov		bx, offset KLUDGES		{ Kludges array }
		mov		dh,[KludMax]            { Length of array }
		mov		dl,0

		mov		StrSiz, KL_SIZ			{ Size of an element of array }
		call	@Comps					{ Compare [si] to array in [bx]}
		mov		al,$01
		jnc		@1a						{ Not found }

		call	@Repl					{ Replace if necessary }

		cmp		Plus,0
		jnz		@YePlus					{ Do not delete end of kludge line }

		push	di      				{ found, look for the end of "string"}
		mov		di,si
		mov		al,$0D
@2:		scasb							{Find NEXT CR}
		loopnz	@2

		mov		si,di					{ Move read ptr to the next "line"}
		pop		di

		jcxz	@Exit					{ End of buffer }

		dec		si
		dec		di
		jmp		@1						{ go on... }

@1bx:	jmp		@1b


@ChkArea:								{ Check for AREA: }
		mov		dh,NetScan
		mov     ScanOn,dh
		mov		bx, offset AREA_ID		{ 'AREA:' }
		mov		dh, AREA_ID_SIZ
		mov		StrSiz,1
		call	@Comps
		jnc		@1bx
		mov		al,Exclude              { Some AREA: found }
		mov     ScanOn,al				{ if EXCLUDE turn scan on, else off }

		mov		bx, offset AREAS		{ Exclude/include areas list }
		mov		dh, [AreaMax]
		or		dh,dh
		jz		@1bx
		mov		StrSiz, AR_SIZ
		push	si
		add		si,AREA_ID_SIZ			{ Skip 'AREA:' }
		call	@Comps
		pop		si
		jnc		@1bx
		mov		al,Exclude				{ If found and exclude turn scan off }
		xor		al,1                    { If found and NOT exclude turn scan on }
		mov     ScanOn,al
		jmp		@1b

@Exit:	sub		di,offset Buf			{ New number of bytes in buffer }
		mov		ax,Actual				{ Save old Actual }
		mov		ActBak,ax
		mov		Actual,di
		jmp		@EndOfProc

@YePlus:
		mov		si,AfterFND_SI
		dec		si
		dec		di
		sub		cx,AfterFND_CX
		mov		Plus,0
		jmp		@1						{ go on... }

@Comps: 		{ Subroutine. }
				{ Compares text at [SI] to the array of PASCAL strings	}
				{ at [BX].  Var StrSiz is the size of an array element }
		push	cx
		mov		ch, 0
@C1:	push	di
		push	si
		mov		cl, byte ptr [bx]		{ Length of text string }
		mov		di, bx					{ string }
		inc		di
		inc		cx
		mov		AfterFND_CX,cx			{ save "end of kludge"}
		repe	cmpsb					{ compare buffer contents with kludge }
		mov		AfterFND_SI,si			{ save "end of kludge"}
		pop		si
		pop		di
		jcxz	@YesRET					{ string found }
		add		bx,StrSiz				{ not found, take next }
		inc		dl
		dec		dh
		jnz		@C1						{ No more strings in array }
		pop		cx
		clc								{ CF=0 : not found }
		retn
@YesRet:
		pop		cx
		inc		dl
		stc								{ CF=1 : found}
		retn

@Repl:			{ Subroutine. }
				{ Looks if REPLACES contains a reference to a substitute }
				{ string.  If so, substitutes source string into ....... }

		mov		bx,offset REPLACES
		mov		dh,0
		dec		dx
		add		bx,dx
		mov		dl,[bx]
		or		dl, dl
		jz		@NoRpl
		dec		dl
		mov		ax,RP_SIZ
		mul		dl

		mov		Plus,0
		push	si
		push	cx
		add		ax, offset RPLSTRINGS
		mov		si,ax
		lodsb
		mov		cl,al
		mov		ch,0
		mov		al,[ds:si]
		cmp		al,'+'
		jnz		@CopyR
		inc		si
		dec		cx
		mov		Plus,1
@CopyR:
{!!!!!!!!!!!!!!!!!!!!!}
		mov		al,01
		stosb
{!!!!!!!!!!!!!!!!!!!!!}
		rep		movsb
		inc		di
		pop		cx
		pop		si
		mov		Replaced,TRUE
@NoRpl:
		retn

@EndOfProc:

end;


		{ Scans and rewrites PKT files }

procedure ScanPKT ( Name:string );
var
	Term, OpenOut, OpenAnyway : boolean;
	Time, TmpAct : longint;
	FileNum : word;
	FT : file;
	Action : string[16];
	Message : string[64];

begin
{$I-}
	OpenOut := FALSE;			{ "Output file opened" }
	OpenAnyway := FALSE;		{ "Open even no kludges found" }

	Assign ( F, PATH+Name );	{ Open source file }
	Reset (F,1);
	IOcheck (F);
	Write (Name);

	Position := 0;				{ Position in the file, longint			}
	Diff := 0;					{ Difference between src & result file	}
	TmpAct := 0;				{ Temp data size in buffer				}
	Shift := 0;					{ Shift from the start of buffer 		}
	FileNum := 1000;			{ Var for temporary filename generation }

	Action := ' Scanned';
	Replaced := FALSE;

	repeat

		Seek (F, Position);		{ Go to the last found CR }
		IOcheck(F);
		BlockRead (F, Buf, BUFSIZ, Actual );
		IOcheck(F);
		if Actual = 0 then break;		{ EOF }
		if Actual = BUFSIZ then begin  		{ FileSize >= BUFSIZ }
			for i := Actual-1 downto 0 do       { Find latest CR }
				if Buf[i] = $0D then break;
			Actual := i+1;
			Position := Position + Actual - 1;	{ Position for the next read }
			Term := FALSE;						{ There's more data in the file }
			OpenAnyway := TRUE;					{ Long file, MUST write...}
		end else begin         			{ Last portion }
			Buf[Actual] := $0D;
			Term := TRUE;
		end;

		ScanBuf;				{ Scan & clean the buffer }

		TotSrc := TotSrc+ActBak;		{ Total (all files) SRC length }
		TmpAct := TmpAct+ActBak;		{ This file SRC length }
		TotDst := TotDst+Actual;		{ Total (all files) DST length }
		Diff := Diff + (ActBak - Actual);	{ This file difference }

			{ Open DST file if necessary }
		if not OpenOut and ( Replaced or OpenAnyway or (ActBak <> Actual) ) then begin
			Assign  ( FO, PATH+'$$PKTCL1.TMP');
			Action := ' Scanned/Written';
			Rewrite ( FO, 1 );
			IOcheck(FO);
			OpenOut := TRUE;
		end;

			{ Write DST file }
		if OpenOut and not (Term and not Replaced and (Diff=0)) then begin
			BlockWrite (FO, Buf[Shift], Actual-Shift );
			IOcheck(FO);
		end;
		Shift := 1;

	until Term;

	if OpenOut then begin;		{ If DST was open... }
	  if Replaced or (Diff>0) then begin  	{ Cleaned : rename DST into SRC name }
		GetFTime ( F,  Time );	IOcheck(F);
		SetFTime ( FO, Time );	IOcheck(FO);
		Close (F);				IOcheck(F);
		Close (FO);				IOcheck(FO);

		repeat					{ Generate a tmp name to rename SRC }
			Str (FileNum, TmpS);
			TmpS := PATH+'ffff'+TmpS+'.pkt';
			Assign ( FT, TmpS  );
			Reset  ( FT );		{ Is there already such file ? }
			IOr := IOresult;
			if IOr = 0 then Close ( FT );	{ Yes; try a new name }
		until IOr <> 0;
		Rename ( F,  TmpS );		IOcheck(F);		{ SRC --> TMP }
		Rename ( FO, PATH+Name );	IOcheck(FO);	{ DST --> SRC }
		Erase  ( F );				IOcheck(F);		{ Erase TMP   }
	  end else begin	{ DST = SRC, erase it }
		Close (F);		IOcheck(F);
		Close (FO);     IOcheck(FO);
		Erase (FO);     IOcheck(FO);
	  end;
	end else begin
		Close (F);
		IOcheck(FO);
	end;

	if Diff > 0 then begin
		TmpAct := (Diff*100 + TmpAct div 2) div TmpAct;
		Action := ' Cleaned ';
		Message := Action+Int2Str(Diff)+' bytes, '+Int2Str(TmpAct)+'%';
	end else if Replaced then
		Message := ' Changed'
	else
		Message := Action;

	WriteLn (Message);
	Log ( ':',Name+Message );
{$I+}

end;


	{ Scan config file }

procedure ScanCFG;
var
	T : text;
	ScanKludges, ScanAreas, Active, StrictNow : boolean;
	KeyWord, Arg1, Arg2 : string[64];
	code : integer;
	TmpSize : word;
	LenArg2 : byte;

	procedure GetArgs;
	var
		p : byte;

		function GetOneArg ( var A : string ) : boolean;
		begin
			if TmpS = '' then begin
				A := '';
				Exit;
			end;
			if TmpS[1] <> '"' then begin
				A := ExtractWord (1, TmpS, DLM);
				p := Length ( A );
				GetOneArg := TRUE;
			end else begin
				Delete ( TmpS, 1, 1 );
				p := Pos ('"', TmpS );
				if p=0 then ErrHalt ( 'Closing <"> not found : '+TmpS );
				A := Copy (TmpS, 1, p-1);
				GetOneArg := FALSE;
			end;
			Delete ( TmpS, 1, p );
			TmpS := Trim (TmpS);
		end;

	begin
		if GetOneArg (Arg1) then
			KeyWord := StUpCase(Arg1)
		else
			KeyWord := '';
		if Arg1[Length(Arg1)] = '#' then Arg1[Length(Arg1)] := #$0D; { Compare literally }
		GetOneArg (Arg2);
	end;

	procedure GetTag;
	begin
		if Arg2 <> '' then Tag := Arg2[1] else Tag := ' ';
		Active := (AddTag and (Tag = ' ')) or (Tag in TagSet);
	end;


begin

	Assign (T, HomePath+HomeName+'.CFG' );
{$I-}
	Reset ( T );
	if IOresult <> 0 then ErrHalt (HomePath+HomeName+'.CFG, file not found' );
{$I+}

	LogFile := '';

	KludMax := 0;	AreaMax := 0;  RplMax := 0;
	ScanKludges := FALSE;
	ScanAreas   := FALSE;
	Exclude := TRUE;
	Tag := ' ';
	while not EOF(T) do begin
		ReadLn ( T, TmpS );
		TmpS := Trim (TmpS);

		if (TmpS[1] <> ';') and (Length(TmpS) > 0) then begin

			GetArgs;

			if KeyWord = '#END' then begin
				ScanKludges := FALSE;
				ScanAreas   := FALSE;

			end else if ScanKludges then begin
			  if Active then begin

				if Length(Arg1) >= KL_SIZ then
					ErrHalt ( 'Kludge "'+Arg1+'" longer than '+Int2Str(KL_SIZ-1)+' bytes');

				Inc(KludMax);
				if KludMax > KL_LEN then
					ErrHalt ( 'Maximum number of kludges ('+Int2Str(KL_LEN)+') exceeded');
				KLUDGES[KludMax] := Arg1;

				if Arg2 <> '' then begin
					Inc(RplMax);
					if RplMax > RP_LEN then
						ErrHalt ( 'Maximum number of replacements ('+Int2Str(RP_LEN)+') exceeded');
					LenArg2 := Length(Arg2);
					if LenArg2 >= AR_SIZ then
						ErrHalt ( 'Replacement "'+Arg1+'" longer than '+Int2Str(AR_SIZ-1)+' bytes');
					if Arg2[1] = '+' then begin
						Dec(LenArg2);
						StrictNow := TRUE;
					end else
						StrictNow := FALSE;
					if LenArg2 > Length(Arg1) then begin
						if StrictCheck or StrictNow then
				ErrHalt  ( 'ERROR : '   +'Replacement "'+Arg2+'" longer than kludge "'+Arg1+'"')
						else
				WarnOnce ( 'WARNING!!! '+'Replacement "'+Arg2+'" longer than kludge "'+Arg1+'"');
					end;
					RPLSTRINGS[RplMax] := Arg2;
					REPLACES [KludMax] := RplMax;
				end;
			  end;
			end else if ScanAreas then begin
			  if Active then begin
				Inc(AreaMax);
				if AreaMax > AR_LEN then
					ErrHalt ( 'Maximum number of areas ('+Int2Str(AR_LEN)+') exceeded');
				if Length(Arg1) >= AR_SIZ then
					ErrHalt ( 'Area name '+Arg1+' longer than '+Int2Str(AR_SIZ-1)+' bytes');
				AREAS[AreaMax] := Arg1;
			  end;
			end else begin
				if          KeyWord = 'LOGFILE' then begin
					LogFile := Arg2;
				end else if KeyWord = 'STRICTCHECK' then begin
					StrictCheck := TRUE;
				end else if KeyWord = 'IFMAILBUGFIX' then begin
					IfmailBugfix := TRUE;
				end else if KeyWord = 'NETSCAN' then begin
					GetTag;
					if Active then NetScan := TRUE;
				end else if KeyWord = '#KLUDGES' then begin
					GetTag;
					ScanKludges := TRUE;
				end else if KeyWord = '#EXCLUDE' then begin
					GetTag;
					ScanAreas := TRUE;
				end else if KeyWord = '#INCLUDE' then begin
					GetTag;
					ScanAreas := TRUE;
					Exclude := FALSE;
				end else begin
					Warn ( 'Unrecognized CFG keyword '+Arg1+'. Ignored');
				end;
			end;
		end;
	end;
	Close(T);

	if KludMax = 0 then ErrHalt ( 'No kludges defined in CFG file' );
end;

procedure GetTags ( S:string );
var
	i : byte;
begin
	for i := 2 to Length (S) do TagSet := TagSet + [UpCase(S[i])];
end;







begin

	Inform ( ID_STR );		{ Message to display }

	AddTag := TRUE;
	TagSet := [];

	if ParamCount > 1 then for i := 2 to ParamCount do begin
		TmpS := ParamStr(i);
		if TmpS[1] = '-' then begin
			AddTag := FALSE;
			GetTags ( TmpS );
		end else if TmpS[1] = '+' then begin
			GetTags ( TmpS );
		end;
	end;

	FSplit (ParamStr(0), HomePath, HomeName, HomeExt);

	LogOpen := FALSE;
	ScanCFG;

	Log ( '+','--- '+ID_STR );

	TmpS := '';
	for i := 1 to ParamCount do
		TmpS := TmpS+ParamStr(i)+' ';
	Log ( ':','Command: '+TmpS );

	if ParamCount < 1 then begin
		WriteLn ( 'Strips garbage kludges from PKT files' );
		WriteLn ( 'Usage:' );
		WriteLn ( '  PKTCLEAN <IN_PKT_PATH> {file_mask} [+|-<kludge_groups>]' );
		WriteLn ( 'if file_mask(s) omitted, *.PKT assumed' );
		WriteLn ( 'CopyLeft 1994 by Dimmy Timchenko, 2:469/15' );
		WriteLn;
		Log ( '!', 'Empty command line, program terminated' );
		CloseLog;
		Halt (2);
	end;

	PATH  := ParamStr(1);

	if PATH[Length(PATH)] <> '\' then PATH := PATH + '\';

	NameMax := 0;
	MaskSpecified := FALSE;

	if ParamCount > 1 then for i := 2 to ParamCount do begin
		TmpS := ParamStr(i);
		if (TmpS[1] <> '-') and (TmpS[1] <> '+') then begin
			ScanDIR ( ParamStr(i) );
			MaskSpecified := TRUE;
		end;
	end;
	if not MaskSpecified then ScanDIR ( '*.PKT' );

	Write ( #$0D );
	TotSrc := 0; TotDst := 0;

	if NameMax = 0 then begin
		Warn ( 'No packets found' );
		CloseLog;
		WriteLn;
		Halt (0);
	end;

		{ Process all files... }
	for NameIdx := 1 to NameMax do
		ScanPKT (Names[NameIdx]);

	TotDiff := (TotSrc-TotDst);
	if TotSrc> 0 then Percent := (TotDiff*100 + TotSrc div 2) div TotSrc
	else Percent := 0;
	Inform ( 'Totals: '+Int2Str(TotDiff div 1024)+'K, '+Int2Str(Percent)+'% saved' );
	CloseLog;
	WriteLn;
	Halt(0);

end.